/*

 Performer file loader that enables level-of-detail switching

 File format:

   filename distance

 where filename is the name of a Performer loadable file and distance is the
 range at which the object disappears and is replaced by the next object in
 the file.  After the last last object's range is exceeded no object is
 displayed.  Distance must be a positive number, and increase with each
 consecutive filename.

 See example.lod for an example file.

 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <Performer/pf.h>
#include <Performer/pf/pfLOD.h>
#include <Performer/pfdu.h>

#include "getString.C"
#include "normalize.C"
#include "parse.C"

extern "C" pfNode * pfdLoadFile_lod (char *filename) {

  pfNotify(PFNFY_NOTICE, PFNFY_PRINT, "pfdLoadFile_lod: %s", filename);
  FILE *fp = fopen(filename, "r");
  if (!fp)
    pfNotify(PFNFY_FATAL, PFNFY_PRINT, 
	     "pfdLoadFile_lod: unable to open %s for reading", filename);


  pfLOD *lodNode ;
  lodNode = new pfLOD ;

  // input line number for error messages
  int lineNum ;
  lineNum = 0 ;

  // number of children of lodNode
  int lodNum ;
  lodNum = 0 ;
  lodNode->setRange(0,   0.0f);  

  // read in the file and store data
  char *line = NULL ;
  while (1) {
    lineNum++ ;
    if (getString(&line, fp)<0) {
      break ;
    } else {
      if (normalize(line)) {
	int argc ;
	char **argv ;
	dpf_charsToArgcArgv(line, &argc, &argv) ;
	if (argc != 2)
	  pfNotify(PFNFY_FATAL, PFNFY_PRINT, 
		   "pfdLoadFile_lod: line %d, %s, invalid number of parameters\n",
		   lineNum, filename);
	else {
	  pfNode *node = pfdLoadFile(argv[0]) ;
	  if (!node)
	    pfNotify(PFNFY_FATAL, PFNFY_PRINT, 
		     "pfdLoadFile_lod: line %d, %s, unable to open %s for reading", 
		     lineNum, filename, argv[0]);
	  else {
	    float range ;
	    if ((sscanf(argv[1],"%f",&range)==1) && (range>=0.f)) {
	      if ((lodNum>0) && (range<=lodNode->getRange(lodNum-1)))
		  pfNotify(PFNFY_FATAL, PFNFY_PRINT, 
			   "pfdLoadFile_lod: line %d, %s, range %f is not greater than previous range %f", 
			   lineNum, filename, range, lodNode->getRange(lodNum-1));
		
	      lodNum++ ;
	      lodNode->setRange(lodNum, range) ;
	      lodNode->addChild(node) ;
	    } else {
	      pfNotify(PFNFY_FATAL, PFNFY_PRINT, 
		       "pfdLoadFile_lod: line %d, %s, invalid range = %s", 
		       lineNum, filename, argv[1]);
	    } //if ((sscanf(argv[1],"%d",&range)==1) && (range>0.f))
	  } //if (!node)
	} //if (argc != 2)
	// how many characters to match
      } //if (normalize(line)) 
    } //if (getString(&line, fp)<0)
  } //while (1) 
  
  // set the center of the lod to be the center of the node's bounding sphere
  pfSphere bsph ;
  lodNode->getBound(&bsph) ;
  lodNode->setCenter(bsph.center) ;

  fclose(fp);
  
  pfNotify(PFNFY_NOTICE, PFNFY_PRINT, "pfdLoadFile_lod: finished reading %s", 
	   filename);

#if 0
  {
    int i ;
    for (i=0; i<lodNode->getNumChildren(); i++) 
      printf("%2.2d: child=%p, range=%f\n",
	     i, lodNode->getChild(i),lodNode->getRange(i+1)) ;
    pfVec3 c ;
    lodNode->getCenter(c) ;
    printf("center = (%f, %f, %f)\n",
	   c[0], c[1], c[2]) ;
  }
#endif

  return((pfNode *) lodNode) ;


}

